/****************************************************************************************************/
/* Extension UI 生成 ライブラリ                                                                     */
/*      Copyright (c) 2012 by FAR/RAKUDA All Rights Reserved                                        */
/*      Original Namespace : LinkRedirector                                                         */
/****************************************************************************************************/
const SELF    = require('self');
const DATA    = require('self').data;
const MOD     = require('page-mod');
const TABS    = require('tabs');
const WIDGET  = require('widget');
const PREFS   = require('simple-prefs');
const STORAGE = require('simple-storage').storage;
var WHO = WHO || 
{
	extension  : require('WHO.extension').extension,
}
	/*******************************************************************************/

	WHO.background = function(para)
	{
		var self = this;

		WHO.extension.storage.setItem('extensionName',para.extensionName);

		this.extensionName = para.extensionName;

		/* サイト別設定の適用範囲 */
		//Jetpackでばぐあり
		// full protocol://domain:port/pathname?query 
		if(para.originType == "full")
		{
			this.originReg = /^.*$/i;
		}
		else
		// page protocol://domain:port/pathname ページレベルで区別
		if(para.originType == "page")
		{
			this.originReg = /^[^\#\?]+/i;
		}
		else
		// port protocol://domain:port/ ポート番号レベルで区別
		if(para.originType == "port")
		{
			this.originReg = /^https?:\/\/[^\/:]+(?:\:\d*)?\//i;
		}
		else
		// protocol protocol://domain プロトコルレベルで区別
		if(para.originType == "protocol")
		{
			this.originReg = /^https?:\/\/[^\/:]+/i;
		}
		else
		// domain domain ドメイン名のみで区別
	//	if(para.originType == "domain")
		{
			this.originReg = /(?!^https?:\/\/)\b[^\/:]+\b/i;
		}

		//------------------------------------------------------------------------------------//
		/* アクティブページ */

		this.focused =
		{
			source      : null,
			tab         : null,
			url         : "",
			origin      : "",
			status      : {},
		//	postMessage : function(){console.log("")}
		};

		//アクティブページが切りかわった時のコールバック
		this.focusedCallback = para.focusedCallback || function(source,origin,message){};
		this.clearedCallback = para.clearedCallback || function(){};
		this.focusedIsLock = false;

		//------------------------------------------------------------------------------------//
		/* ボタン・バッヂ・ポップアップの作成 */

		para.button=para.button||{};

		this.defaultButton =
		{
			title : para.button.title || "Disabled",
			icon  : para.button.icon  || "icons/icon18.png"
		}

		if(para.popup)
		{
			WHO.extension.setPopupSize(para.popup.width  || 320,para.popup.height || 500);
		}

		/* オプション */
		this.options = WHO.extension.createPopup({href:"options.html",width:640,height:800});

		/* ポップアップ */
		this._popup = para.popup;
	//	this.popup  = this._createPopup(this._popup);

		/* ボタン */
		this._button =
		{
			id        : "popupbutton",
			label     : this.extensionName,
			tooltip   : this.defaultButton.title,
			contentURL: DATA.url(this.icon),
		//	panel     : this.popup,
		//	onClick   : function(){}
		}
		this.disabledButton = true;

		//------------------------------------------------------------------------------------//
		/* タブ移動やページ移動時の初期化処理 */

		/* ページにフォーカスした時 */
		//現在のアクティブページを登録
		WHO.extension.addMessageListener('FocusedPage',function(port,message)
		{
			if(message.method == "set")
			{
				if(message.focused || WHO.extension.isFocusedTab(port.tab))
				{
					self.setFocused(port,message);
				}
			}
		});
		/* ポップアップにアクティブページの情報を送る */
		WHO.extension.addRequestListener('FocusedPage',function(sender,message)
		{
			if(message.method == "get")
			{
				if(self.focused.source)
				{
					sender.messageCallback({method:"set",focused:{url:self.focused.url,origin:self.focused.origin,status:self.focused.status}});
				}
			}
		});

		/* ボタンの表示設定の切り替え */
		var view = PREFS.prefs.buttonConfig;
		WHO.extension.storage.setItem('buttonConfig',JSON.stringify({view:view}));
		PREFS.on("buttonConfig",function(e)
		{
			var view = PREFS.prefs.buttonConfig;
			WHO.extension.storage.setItem('buttonConfig',JSON.stringify({view:view}));
			self.toggleButton();
		});

		this.toggleButton();//起動時

		/* ボタンの初期化 */
		var urltest = /^https?:\/\//i;
		/* タブを移動したとき */
		TABS.on('activate',function(tab)
		{
			if(!tab||!urltest.test(tab.url))
			{
				self.clearFocused();
			}
		});

		//------------------------------------------------------------------------------------//

		return this;
	};



	/**************************************************************************************/

	//------------------------------------------------------------------------------------//
	/* 設定ページを開く */
	WHO.background.prototype.openOptions = function(hash)
	{
		this.options.show();
	};

	//------------------------------------------------------------------------------------//

	WHO.background.prototype._createPopup = function(para)
	{
		var self = this;
		if(para)
		{
			para.onShow = function(w)
			{
				//ボタンが無効時
				if(self.disabledButton)
				{
					this.hide();
				}
				else
				{
					this.port.emit('storage',STORAGE);
					this.port.emit('show',"");
				}
			};
			var popup  = WHO.extension.createPopup(para);
			popup.port.on('options',function(message)
			{
				self.options.show();
				self.options.port.emit('open',message);
			});
			return popup;
		}
		return null;
	};

	//------------------------------------------------------------------------------------//
	/* ボタン表示を切り替える */

	WHO.background.prototype.toggleButton = function()
	{
	//	var buttonConfig = JSON.parse(WHO.extension.storage.getItem('buttonConfig'))||{view:true};
		var view = PREFS.prefs.buttonConfig;
		if(view)
		{
			var popup  = this._createPopup(this._popup);
			this._button.panel = popup;
			this.popup =
			{
				source : popup,
				tab    : popup.tab,
				url    : popup.url,
				postMessage : function(type,message)
				{
					popup.port.emit('message',
					{
						type       :type,
						message    :message
					});
				}
			};
			this.button = WIDGET.Widget(this._button);
		}
		else
		{
			if(this.button)
			{
				this.button.destroy();
			}
		}
	};

	/* ボタンを更新する */

	WHO.background.prototype.updateButton = function(para)
	{
		this._button.tooltip    = para.title || this.defaultButton.title;
		this._button.contentURL = DATA.url(para.icon  || this.icon);
		if(this.button)
		{
			this.button.tooltip    = para.title || this.defaultButton.title;
			this.button.contentURL = DATA.url(para.icon  || this.icon);
		}
	};

	//------------------------------------------------------------------------------------//
	/* バッジを更新する */

	WHO.background.prototype.updateBadge = function(para)
	{
	//	if(window.opera)
	//	{
	//		this.button.badge.display     = para.display||"none";
	//		this.button.badge.textContent = para.text||"-";
	//	}
	};

	/**************************************************************************************/

	//------------------------------------------------------------------------------------//
	/* フォーカスの初期化 */

	WHO.background.prototype.clearFocused = function()
	{
		if(!this.focusedIsLock)
		{
			this.focused =
			{
				source      : null,
				tab         : null,
				url         : "",
				origin      : "",
				status      : {},
			//	postMessage : function(){console.log("")}
			};
			if(this.button)
			{
				this.disabledButton = true;
				this.updateButton(this.defaultButton);
			}
			this.clearedCallback();
		}
	};

	/* 現在のフォーカスを登録 */

	WHO.background.prototype.setFocused = function(port,message)
	{
		if(!this.focusedIsLock && this.focused.source != port.source && this.originReg.test(port.url))
		{
			var origin = port.url.match(this.originReg)[0];
			this.focused =
			{
				source      : port.source,
				tab         : port.tab,
				url         : port.url,
				origin      : origin,
				status      : {},
				postMessage : port.postMessage
			};
			if(this.button)
			{
				this.disabledButton = false;
				if(this.badge)
				{
					this.updateBadge({display:"block"});
				}
			}
			this.focusedCallback(message);
		}
	};

	//------------------------------------------------------------------------------------//
	/* 現在のフォーカスをロック */

	WHO.background.prototype.lockFocused = function()
	{
		this.focusedIsLock = true;
	};

	/* 現在のフォーカスをアンロック */

	WHO.background.prototype.unlockFocused = function()
	{
		this.focusedIsLock = false;
	};

	//------------------------------------------------------------------------------------//
	/*  */

	WHO.background.prototype.getOrigin = function(url)
	{
		if(this.originReg.test(url))
		{
			var origin = url.match(this.originReg)[0];
			return origin;
		}
		else
		{
			return "";
		}
	};

	/**************************************************************************************/

	exports.background = WHO.background;
