/* Firefox(Jetpack) Add-on 側のライブラリ */

const DATA    = require('self').data;
const PANEL   = require('panel');
const TABS    = require('tabs');
const STORAGE = require('simple-storage').storage;

var WHO = WHO || {};

	/**************************************************************************************/

	WHO.extension = WHO.extension || {};
	WHO.extension.isBackground = true;

	//------------------------------------------------------------------------------------//
	/* ポップアップのサイズを調整 */
	//background 側

	WHO.extension.setPopupSize = function(width,height)
	{
		WHO.extension.popupSize =
		{
			width : width,
			height: height
		};
	};

	//------------------------------------------------------------------------------------//

	/**************************************************************************************/

	/* ###### ### #####  ###### ######  #####  #     # */
	/* #       #  #    # #      #      #     #  #   #  */
	/* #       #  #    # #      #      #     #   # #   */
	/* #####   #  #    # #####  #####  #     #    #    */
	/* #       #  #####  #      #      #     #   # #   */
	/* #       #  #  #   #      #      #     #  #   #  */
	/* #      ### #   ## ###### #       #####  #     # */

	WHO.extension.isJetpack = true;

	//------------------------------------------------------------------------------------//
	/* ストレージ */

	WHO.extension.storage = 
	{
		setItem : function(key,value)
		{
			STORAGE[key] = value;
		},
		getItem : function(key)
		{
			return STORAGE[key]||null;
		},
		removeItem : function(key)
		{
			STORAGE[key] = null;
		}
	};

	//------------------------------------------------------------------------------------//
	/* 多言語化ファイル */

	WHO.extension._localefiles = {};
	WHO.extension.setLocaleFiles = function(files)
	{
		WHO.extension._localefiles = files;
	};
	WHO.extension.getDataURL = function(path)
	{
		return DATA.url(WHO.extension._localefiles[path]||path);
	};
	WHO.extension.loadData = function(path)
	{
		return DATA.load(WHO.extension._localefiles[path]||path);
	};

	//------------------------------------------------------------------------------------//
	/* タブがアクティブなタブかどうか */
	WHO.extension.isFocusedTab = function(tab)
	{
		return (tab && tab == TABS.activeTab);
	};

	//------------------------------------------------------------------------------------//
	/* ポップアップの作成 */

	WHO.extension.createPopup = function(para)
	{
		WHO.extension.setPopupSize(para.width || 320,para.height || 500);
		var html    = WHO.extension.loadData(para.href||"popup.html");

		//ページのスクリプトを解析して
		var scripts = html.split(/(?:^|")[\s\S]*?(?:<script [^"]*src="|$)/gi)||[];
			scripts.pop();
			scripts.shift();
			for(var i = 0;i < scripts.length;i++)
			{
				scripts[i] = WHO.extension.getDataURL(scripts[i]);
			}

		var popup = PANEL.Panel(
		{
			width : para.width  || 320,
			height: para.height || 500,
			allow : {script:false},//ページのスクリプトは実行しない
			contentURL: WHO.extension.getDataURL(para.href || "popup.html"),
			contentScriptWhen: "start",
			contentScriptFile:scripts,
			onShow : para.onShow || function(w)
			{
				popup.port.emit('storage',STORAGE);
				popup.port.emit('show',"");
			},
			onHide : para.onHide || function()
			{
				popup.port.emit('hide',"");
			}
		});
		popup.port.on('hide',function(){popup.hide();});

		WHO.extension.onAttach(popup,true);
		return popup;
	};

	/**************************************************************************************/

	WHO.extension._listeners = 
	{
		"message":{},
		"request":{},
	};

	WHO.extension._workers = [];
	WHO.extension.onAttach = function(worker,keep)
	{
		if(keep)
		{
			//ストレージデータを送る
			worker.port.emit('storage',STORAGE);
			WHO.extension._workers.push(worker);
		}

		worker.port.on('setItem',function(data)
		{
			STORAGE[data.key] = data.value;
		});
		worker.port.on('removeItem',function(data)
		{
			STORAGE[data.key] = null;
		});


		/* 永続メッセージ接続の取得 */
		var listeners = WHO.extension._listeners.message||{};
		for(var type in listeners)
		{
			for(var i = 0,listener;listener = listeners[type][i];i++)
			{
				worker.port.on('message',WHO.extension._addMessageListener(type,listener,worker));
			}
		}
		/* background の単発メッセージ受信 */
		var listeners  = WHO.extension._listeners.request||{};
		for(var type in listeners)
		{
			for(var i = 0,listener;listener = listeners[type][i];i++)
			{
				worker.port.on('request',WHO.extension._addRequestListener(type,listener,worker));
			}
		}
	};
	WHO.extension._addMessageListener = function(type,listener,worker)
	{
		return function(data)
		{
			if(data.type == type)
			{
				var connection =
				{
					source      : worker,
					tab         : worker.tab,
					url         : worker.url,
					postMessage : function(type,message)
					{
						worker.port.emit('message',
						{
							type       :type,
							message    :message
						});
					}
				};

				listener(connection,data.message);
			}
		};
	};
	WHO.extension._addRequestListener = function(type,listener,worker)
	{
		return function(data)
		{
			//console.log(type+":"+data.messageType+":"+JSON.stringify(data.message));
			if(data.type == type)
			{
				var sender =
				{
					source      : worker,
					tab         : worker.tab,
					url         : worker.url,
					messageCallback : function(){}
				}

				if(data.requestID)
				{
					sender.messageCallback = function(message)
					{
						//console.log("send response:"+type+":");
						worker.port.emit('response',
						{
							type       :type,
							responseID :data.requestID,
							message    :message
						});
					}
				};
				listener(sender,data.message);
			}
		};
	};

	//------------------------------------------------------------------------------------//
	/* 永続メッセージ接続の取得 */

	WHO.extension.addMessageListener = function(type,listener)
	{
		WHO.extension._listeners.message[type]=WHO.extension._listeners.message[type]||[];
		WHO.extension._listeners.message[type].push(listener);

		//既に生成されているポート
		for(var i = 0,worker;worker = WHO.extension._workers[i];i++)
		{
			worker.port.on('message',WHO.extension._addMessageListener(type,listener,worker));
		}
	};

	/* 永続メッセージ接続の取得を削除 */

	WHO.extension.removeMessageListener = function(type,listener)
	{
		var listeners = WHO.extension._listeners.message[type];
		for(var i = listeners.length - 1;i >= 0;i--)
		{
			if(listener === listeners[i])
			{
				listeners.splice(i,1);
			}
		}
	};

	//------------------------------------------------------------------------------------//
	/* background の単発メッセージ受信 */

	WHO.extension.addRequestListener = function(type,listener)
	{
		WHO.extension._listeners.request[type]=WHO.extension._listeners.request[type]||[];
		WHO.extension._listeners.request[type].push(listener);

		//既に生成されているポート
		for(var i = 0,worker;worker = WHO.extension._workers[i];i++)
		{
			worker.port.on('request',WHO.extension._addRequestListener(type,listener,worker));
		}
	};

	/* 単発メッセージ受信を削除 */

	WHO.extension.removeRequestListener = function(type,listener)
	{
		var listeners = WHO.extension._listeners.request[type];
		for(var i = listeners.length - 1;i >= 0;i--)
		{
			if(listener === listeners[i])
			{
				listeners.splice(i,1);
			}
		}
	};

//	//------------------------------------------------------------------------------------//
//	/* Tab へ単発メッセージ */
//
//	WHO.extension.postMessageToContent = function(tab,type,message)
//	{
//		//未実装
//	};

	/**************************************************************************************/

	exports.extension = WHO.extension;
